package io.tswf.transmutations.expansion

import groovy.transform.CompileStatic
import io.tswf.transmutations.common.annotation.ClassReference
import io.tswf.transmutations.expansion.annotations.Expand
import io.tswf.transmutations.expansion.generator.ExpansionInjectionContext
import io.tswf.transmutations.expansion.generator.ExpansionInjector
import io.tswf.transmutations.expansion.reader.ExpansionMetadataReader

@CompileStatic
class ExpansionApp {
    static void main(String[] args) {
        def expansionClass = LengthCalculatorExpansion.toClassNode()
        def expansionMeta = ExpansionMetadataReader.readExpansionMetadata(expansionClass)
        def expansionTarget = ClassLoader.contextClassloader().loadClass(expansionMeta.targets.first().className).toClassNode()

        ExpansionInjector.injectExpansion(new ExpansionInjectionContext(
                targetClass: expansionTarget,
                expansionMeta: expansionMeta
        ))

        new File("expansion-class.class").bytes = expansionTarget.toByteArray()
    }
}

interface ILengthCalculator {
    int getLength(String stringParam)
}

@Expand(targets = @ClassReference(PreviouslyNotLengthCalculator))
class LengthCalculatorExpansion implements ILengthCalculator{

    @Override
    int getLength(String stringParam) {
        return stringParam?.length() ?: 0
    }
}

class TestClassToExpand {

}
