package io.tswf.transmutations.expansion.exception

import groovy.transform.CompileStatic
import io.tswf.transmutations.expansion.meta.ExpansionMetadata
import org.objectweb.asm.tree.MethodNode

@CompileStatic
class ExpansionMethodMetadataReadingException extends RuntimeException {
    ExpansionMetadata expansionMeta
    MethodNode readingMethod

    ExpansionMethodMetadataReadingException(ExpansionMetadata expansionMeta, MethodNode readingMethod) {
        super(getExceptionMessage("", expansionMeta, readingMethod))
        this.expansionMeta = expansionMeta
        this.readingMethod = readingMethod
    }

    ExpansionMethodMetadataReadingException(String message, ExpansionMetadata expansionMeta, MethodNode readingMethod) {
        super(getExceptionMessage(message, expansionMeta, readingMethod))
        this.expansionMeta = expansionMeta
        this.readingMethod = readingMethod
    }

    ExpansionMethodMetadataReadingException(String message, Throwable cause, ExpansionMetadata expansionMeta, MethodNode readingMethod) {
        super(getExceptionMessage(message, expansionMeta, readingMethod), cause)
        this.expansionMeta = expansionMeta
        this.readingMethod = readingMethod
    }

    ExpansionMethodMetadataReadingException(Throwable cause, ExpansionMetadata expansionMeta, MethodNode readingMethod) {
        super(getExceptionMessage("", expansionMeta, readingMethod), cause)
        this.expansionMeta = expansionMeta
        this.readingMethod = readingMethod
    }

    static String getExceptionMessage(String message, ExpansionMetadata expansionMeta, MethodNode readingMethod) {
        return "Exception occurred while trying to read expansion method '${readingMethod.javaFullName}' located in '${expansionMeta.expansionClass.javaName}': ${message}"
    }
}
