package io.tswf.transmutations.expansion.reader

import groovy.transform.CompileStatic
import io.tswf.transmutations.common.conflict.ConflictStrategyReader
import io.tswf.transmutations.expansion.annotations.Ignore
import io.tswf.transmutations.expansion.exception.InvalidExpansionMethodException
import io.tswf.transmutations.expansion.meta.ExpansionMetadata
import io.tswf.transmutations.expansion.meta.ExpansionMethodMetadata
import io.tswf.transmutations.expansion.util.ReservedNames
import org.objectweb.asm.tree.MethodNode

import java.lang.reflect.Modifier

import static org.objectweb.asm.Opcodes.ACC_SYNTHETIC

@CompileStatic
class ExpansionMethodMetadataReader {

    static ExpansionMethodMetadata readExpansionMethodMetadata(
            ExpansionMetadata expansionMetadata,
            MethodNode expansionMethod
    ) {
        def result = new ExpansionMethodMetadata()
        result.expansionMethod = expansionMethod
        result.expansion = expansionMetadata

        def isExpansionMethodSynthetic = (expansionMethod.access & ACC_SYNTHETIC) != 0

        if (isExpansionMethodSynthetic) {
            throw new InvalidExpansionMethodException("expansion methods can not be synthetic", result)
        }

        if (expansionMethod.hasAnnotation(Ignore)) {
            throw new InvalidExpansionMethodException("this expansion method ignored by @${Ignore.simpleName} annotation!", result)
        }

        if (Modifier.isStatic(expansionMethod.access)) {
            throw new InvalidExpansionMethodException("expansion methods can not be static", result)
        }

        if (!Modifier.isPublic(expansionMethod.access)) {
            throw new InvalidExpansionMethodException("expansion methods must be public", result)
        }

        if (expansionMethod.name in ReservedNames.CLASS_INTERNAL_METHOD_NAMES) {
            throw new InvalidExpansionMethodException("name '${expansionMethod.name}' is reserved and can not be accessed from expansions", result)
        }

        result.conflictStrategy = ConflictStrategyReader.readConflictStrategy(expansionMethod)

        return result
    }

}
